/*
 * SPDX-FileCopyrightText: 2018, 2020 Bruce Cowan <bruce@bcowan.me.uk>
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#include "glob-match-widget.h"

struct _GlobMatchWidget
{
    GtkListBoxRow parent_instance;

    GlobMatchItem *item;

    GtkWidget *label;
};

G_DEFINE_TYPE (GlobMatchWidget, glob_match_widget, GTK_TYPE_LIST_BOX_ROW)

// Pango doesn't support the cleanup functions
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PangoAttrList, pango_attr_list_unref)

enum
{
    PROP_0,
    PROP_ITEM,
    N_PROPS
};

static GParamSpec *properties[N_PROPS];

static void
glob_match_widget_dispose (GObject *object)
{
    GlobMatchWidget *self = GLOB_MATCH_WIDGET (object);

    g_clear_object (&self->item);

    G_OBJECT_CLASS (glob_match_widget_parent_class)->dispose (object);
}

static void
glob_match_widget_get_property (GObject    *object,
                                unsigned    prop_id,
                                GValue     *value,
                                GParamSpec *pspec)
{
    GlobMatchWidget *self = GLOB_MATCH_WIDGET (object);

    switch (prop_id)
      {
      case PROP_ITEM:
          g_value_set_object (value, glob_match_widget_get_item (self));
          break;
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      }
}

static void
glob_match_widget_set_property (GObject      *object,
                                unsigned      prop_id,
                                const GValue *value,
                                GParamSpec   *pspec)
{
    GlobMatchWidget *self = GLOB_MATCH_WIDGET (object);

    switch (prop_id)
      {
      case PROP_ITEM:
          // No need to clear object because it can only be set once anyway
          self->item = g_value_dup_object (value);
          break;
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      }
}

static void
glob_match_widget_constructed (GObject *object)
{
    GlobMatchWidget *self = GLOB_MATCH_WIDGET (object);

    G_OBJECT_CLASS (glob_match_widget_parent_class)->constructed (object);

    const char *text = glob_match_item_get_word (self->item);
    gtk_label_set_text (GTK_LABEL (self->label), text);

    // Highlighting
    GMatchInfo *match = glob_match_item_get_match (self->item);

    if (match)
    {
        int start, end = 0;
        g_match_info_fetch_pos (match, 0, &start, &end);

        g_autoptr (PangoAttrList) list = pango_attr_list_new ();
        PangoAttribute *bold = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
        bold->start_index = start;
        bold->end_index = end;
        pango_attr_list_insert (list, bold);

        gtk_label_set_attributes (GTK_LABEL (self->label), list);
    }
}

static void
glob_match_widget_class_init (GlobMatchWidgetClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

    object_class->constructed = glob_match_widget_constructed;
    object_class->dispose = glob_match_widget_dispose;
    object_class->get_property = glob_match_widget_get_property;
    object_class->set_property = glob_match_widget_set_property;

    properties[PROP_ITEM] = g_param_spec_object ("item", "Item", "Match item",
                                                 GLOB_TYPE_MATCH_ITEM,
                                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);

    g_object_class_install_properties (object_class, N_PROPS, properties);

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 "/uk/me/bcowan/wordsearch/match-widget.ui");
    gtk_widget_class_bind_template_child (widget_class, GlobMatchWidget, label);
}

static void
glob_match_widget_init (GlobMatchWidget *self)
{
    gtk_widget_init_template (GTK_WIDGET (self));
}

/**
 * glob_match_widget_new:
 * @item: The #GlobMatchItem to be represented
 *
 * Creates a new widget to display @item.
 *
 * Returns: a new #GlobMatchWidget
 */
GtkWidget *
glob_match_widget_new (GlobMatchItem *item)
{
    return g_object_new (GLOB_TYPE_MATCH_WIDGET,
                         "item", item,
                         NULL);
}

/**
 * glob_match_widget_get_item:
 * @self: a #GlobMatchWidget
 *
 * Gets the #GlobMatchItem this widget is representing.
 *
 * Returns: (transfer none): the #GlobMatchItem
 */
GlobMatchItem *
glob_match_widget_get_item (GlobMatchWidget *self)
{
    g_assert (GLOB_IS_MATCH_WIDGET (self));

    return self->item;
}
